/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "houdini/houdini.pmh"

namespace fe
{

String SurfaceAccessorHoudini::type(void) const
{
	if(!m_attrType.empty())
	{
		return m_attrType;
	}

	String attrType ="integer";

#ifdef FE_HOUDINI_USE_GA
	if(m_handle.isSharedStringAttribute())
	{
		attrType="string";
	}
#else
	if(m_handle.isIndexAttribute())
	{
		attrType="string";
	}
#endif

	if(m_handle.isFloatOrVectorAttribute())
	{
		const int floatCount=m_handle.getFloatCount(false);

		switch(floatCount)
		{
			case 1:
				attrType="real";
				break;
			case 2:
				attrType="vector2";
				break;
			case 3:
				attrType="vector3";
				break;
			case 4:
				attrType="vector4";
				break;
			default:
				;
		}
	}

	const_cast<SurfaceAccessorHoudini*>(this)->m_attrType=attrType;

	return m_attrType;
}

void SurfaceAccessorHoudini::addString(void)
{
//	feLog("SurfaceAccessorHoudini::addInteger \"%s\" %p\n",
//			m_attrName.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_detail)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addStringTuple(GA_ATTRIB_DETAIL,
					m_attrName.c_str(), 1);
#else
			m_pGdpChangeable->addAttrib(m_attrName.c_str(),
					sizeof(UT_String),GB_ATTRIB_INDEX,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_primitive)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addStringTuple(GA_ATTRIB_PRIMITIVE,
					m_attrName.c_str(), 1);
#else
			m_pGdpChangeable->addPrimAttrib(m_attrName.c_str(),
					sizeof(UT_String),GB_ATTRIB_INDEX,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addStringTuple(GA_ATTRIB_POINT,
					m_attrName.c_str(), 1);
#else
			m_pGdpChangeable->addPointAttrib(m_attrName.c_str(),
					sizeof(UT_String),GB_ATTRIB_INDEX,NULL);
#endif
		}
		bindInternal(m_element,m_attrName);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addString"
			" could not add attribute \"%s\"\n",m_attrName.c_str());
		return;
	}
}
void SurfaceAccessorHoudini::addInteger(void)
{
//	feLog("SurfaceAccessorHoudini::addInteger \"%s\" %p\n",
//			m_attrName.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_detail)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addIntTuple(GA_ATTRIB_DETAIL,
					m_attrName.c_str(), 1);
#else
			m_pGdpChangeable->addAttrib(m_attrName.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_primitive)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addIntTuple(GA_ATTRIB_PRIMITIVE,
					m_attrName.c_str(), 1);
#else
			m_pGdpChangeable->addPrimAttrib(m_attrName.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addIntTuple(GA_ATTRIB_POINT,
					m_attrName.c_str(), 1);
#else
			m_pGdpChangeable->addPointAttrib(m_attrName.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
#endif
		}
		bindInternal(m_element,m_attrName);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addInteger"
			" could not add attribute \"%s\"\n",m_attrName.c_str());
		return;
	}
}

void SurfaceAccessorHoudini::addReal(void)
{
//	feLog("SurfaceAccessorHoudini::addReal \"%s\" %p\n",
//			m_attrName.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_detail)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_DETAIL,
					m_attrName.c_str(), 1);
#else
			const Real zero=0.0;
			m_pGdpChangeable->addAttrib(m_attrName.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_primitive)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_PRIMITIVE,
					m_attrName.c_str(), 1);
#else
			const Real zero=0.0;
			m_pGdpChangeable->addPrimAttrib(m_attrName.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#ifdef FE_HOUDINI_USE_GA
			m_pGdpChangeable->addFloatTuple(GA_ATTRIB_POINT,
					m_attrName.c_str(), 1);
#else
			const Real zero=0.0;
			m_pGdpChangeable->addPointAttrib(m_attrName.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
#endif
		}
		bindInternal(m_element,m_attrName);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addReal"
			" could not add attribute \"%s\"\n",m_attrName.c_str());
		return;
	}
}

void SurfaceAccessorHoudini::addSpatialVector(void)
{
//	feLog("SurfaceAccessorHoudini::addSpatialVector \"%s\" %p\n",
//			m_attrName.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
#ifdef FE_HOUDINI_USE_GA
		GA_RWAttributeRef attrRef;
#endif
		if(m_element==SurfaceAccessibleI::e_detail)
		{
#ifdef FE_HOUDINI_USE_GA
			attrRef=m_pGdpChangeable->addFloatTuple(GA_ATTRIB_DETAIL,
					m_attrName.c_str(), 3);
#else
			m_pGdpChangeable->addAttrib(m_attrName.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_primitive)
		{
#ifdef FE_HOUDINI_USE_GA
			attrRef=m_pGdpChangeable->addFloatTuple(GA_ATTRIB_PRIMITIVE,
					m_attrName.c_str(), 3);
#else
			m_pGdpChangeable->addPrimAttrib(m_attrName.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
#endif
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
#ifdef FE_HOUDINI_USE_GA
			attrRef=m_pGdpChangeable->addFloatTuple(GA_ATTRIB_POINT,
					m_attrName.c_str(), 3);
#else
			m_pGdpChangeable->addPointAttrib(m_attrName.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
#endif
		}
#ifdef FE_HOUDINI_USE_GA
		if(attrRef.isValid())
		{
			if(m_attrName=="N")
			{
				attrRef.getAttribute()->setTypeInfo(GA_TYPE_NORMAL);
			}
			else if(m_attrName=="Cd")
			{
				attrRef.getAttribute()->setTypeInfo(GA_TYPE_COLOR);
			}
			else
			{
				attrRef.getAttribute()->setTypeInfo(GA_TYPE_VECTOR);
			}
		}
#endif
		bindInternal(m_element,m_attrName);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addSpatialVector"
			" could not add attribute \"%s\"\n",m_attrName.c_str());
		return;
	}
}

BWORD SurfaceAccessorHoudini::bindInternal(
	SurfaceAccessibleI::Element a_element,const String& a_name)
{
	FEASSERT(m_pGdp);
	m_element=SurfaceAccessibleI::e_point;
	if(a_element>=0 && a_element<=5)
	{
		m_element=a_element;
	}
	m_attrName=a_name;
	m_handle.invalidate();
	m_pPrimitive=NULL;
	m_pGroup=NULL;
	m_groupIndex=0;

#ifdef FE_HOUDINI_USE_GA
	m_pIterator=NULL;
#else
	m_pPointChangeable=NULL;
	m_pPoint=NULL;
#endif

	if(m_element==SurfaceAccessibleI::e_primitiveGroup && m_attrName.empty())
	{
		m_element=SurfaceAccessibleI::e_primitive;
		m_attribute=SurfaceAccessibleI::e_vertices;
		return TRUE;
	}

	switch(m_element)
	{
		case SurfaceAccessibleI::e_point:
			if(m_pGdpChangeable)
			{
				m_handle=m_pGdpChangeable->getPointAttribute(a_name.c_str());
			}
			else
			{
				m_handle=m_pGdp->getPointAttribute(a_name.c_str());
			}
			break;

		case SurfaceAccessibleI::e_pointGroup:
			if(m_pGdpChangeable)
			{
				m_pGroup=m_pGdpChangeable->findPointGroup(m_attrName.c_str());
			}
			else
			{
#ifdef FE_HOUDINI_USE_GA
				m_pGroup=const_cast<GA_PointGroup*>(
						m_pGdp->findPointGroup(m_attrName.c_str()));
#else
				m_pGroup=const_cast<GB_PointGroup*>(
						m_pGdp->findPointGroup(m_attrName.c_str()));
#endif
			}
			break;

		case SurfaceAccessibleI::e_vertex:
			break;

		case SurfaceAccessibleI::e_primitive:
			if(m_pGdpChangeable)
			{
				m_handle=m_pGdpChangeable->getPrimAttribute(a_name.c_str());
			}
			else
			{
				m_handle=m_pGdp->getPrimAttribute(a_name.c_str());
			}
			break;

		case SurfaceAccessibleI::e_primitiveGroup:
			if(m_pGdpChangeable)
			{
				m_pGroup=m_pGdpChangeable->findPrimitiveGroup(
						m_attrName.c_str());
			}
			else
			{
#ifdef FE_HOUDINI_USE_GA
				m_pGroup=const_cast<GA_PrimitiveGroup*>(
						m_pGdp->findPrimitiveGroup(m_attrName.c_str()));
#else
				m_pGroup=const_cast<GB_PrimitiveGroup*>(
						m_pGdp->findPrimitiveGroup(m_attrName.c_str()));
#endif
			}
			break;

		case SurfaceAccessibleI::e_detail:
			if(m_pGdpChangeable)
			{
				m_handle=m_pGdpChangeable->getDetailAttribute(a_name.c_str());
				m_handle.setElement(m_pGdpChangeable);
			}
			else
			{
				m_handle=m_pGdp->getDetailAttribute(a_name.c_str());
				m_handle.setElement(m_pGdp);
			}
			break;

		default:
			;
	}

	return isBound();
}

void SurfaceAccessorHoudini::harden(void)
{
#if FE_HOUDINI_HARDENING
	if(m_pGdpChangeable)
	{
		sp<SurfaceAccessibleHoudini> spAccessible=m_spSurfaceAccessibleI;
		if(spAccessible.isNull())
		{
			return;
		}

		String attrRate;
		switch(m_element)
		{
			case SurfaceAccessibleI::e_point:
				attrRate="t";
				break;
			case SurfaceAccessibleI::e_pointGroup:
				attrRate="T";
				break;
			case SurfaceAccessibleI::e_vertex:
				attrRate="v";
				break;
			case SurfaceAccessibleI::e_primitive:
				attrRate="r";
				break;
			case SurfaceAccessibleI::e_primitiveGroup:
				attrRate="R";
				break;
			case SurfaceAccessibleI::e_detail:
				attrRate="d";
				break;
		}

		const String attrKey=attrRate+":"+m_attrName;

		m_spHardening=spAccessible->hardening(attrKey);

		if(!m_spHardening->hardened())
		{
			GA_RWAttributeRef attrRef;

			if(m_attribute==SurfaceAccessibleI::e_vertices)
			{
				attrRef=m_pGdpChangeable->findPointAttribute("P");
			}
			else
			{
				GA_AttributeOwner owner=m_handle.getDictionary();
				attrRef=m_pGdpChangeable->findAttribute(
						owner,m_attrName.c_str());
			}

			GA_Attribute* pAttribute=attrRef.getAttribute();

			if(pAttribute)
			{
				m_spHardening->harden(
						new GA_AutoHardenForThreading(*pAttribute));
			}
		}
	}
#endif
}

void SurfaceAccessorHoudini::select(U32 a_index)
{
	FEASSERT(m_pGdp);
	switch(m_element)
	{
		case SurfaceAccessibleI::e_point:
			if(m_pGdpChangeable)
			{
#ifdef FE_HOUDINI_USE_GA
//				GEO_Point* pPointChangeable=m_pGdpChangeable->getGEOPoint(
//						m_pGdpChangeable->pointOffset(a_index));
//				m_handle.setElement(pPointChangeable);
				m_handle.setPoint(m_pGdpChangeable->pointOffset(a_index));
#else
				m_pPointChangeable=m_pGdpChangeable->points()(a_index);
				m_handle.setElement(m_pPointChangeable);
				m_pPoint=m_pPointChangeable;
#endif
			}
			else
			{
#ifdef FE_HOUDINI_USE_GA
//				const GEO_Point* pPoint=m_pGdp->getGEOPoint(
//						m_pGdp->pointOffset(a_index));
//				m_handle.setElement(pPoint);
				m_handle.setPoint(m_pGdp->pointOffset(a_index));
#else
				m_pPoint=m_pGdp->points()(a_index);
				m_handle.setElement(m_pPoint);
				m_pPointChangeable=NULL;
#endif
			}
			break;

		case SurfaceAccessibleI::e_primitive:
			if(m_pGdpChangeable)
			{
#ifdef FE_HOUDINI_USE_GA
				GEO_Primitive* pPrimitive=m_pGdpChangeable->getGEOPrimitive(
						m_pGdpChangeable->primitiveOffset(a_index));

//				m_handle.setPrimitive(
//						m_pGdpChangeable->primitiveOffset(a_index));
#else
				GEO_Primitive* pPrimitive=
						m_pGdpChangeable->primitives()(a_index);
#endif
				m_pPrimitive=pPrimitive; //* to const
				m_handle.setElement(pPrimitive);
			}
			else
			{
#ifdef FE_HOUDINI_USE_GA
				m_pPrimitive=m_pGdp->getGEOPrimitive(
						m_pGdp->primitiveOffset(a_index));

//				m_handle.setPrimitive(m_pGdp->primitiveOffset(a_index));
#else
				m_pPrimitive=m_pGdp->primitives()(a_index);
#endif
				m_handle.setElement(m_pPrimitive);
			}
			break;

//		case SurfaceAccessibleI::e_vertex:
//			break;

		case SurfaceAccessibleI::e_detail:
			//* index not relevant
			break;

		case SurfaceAccessibleI::e_pointGroup:
		{
#ifdef FE_HOUDINI_USE_GA
			if(m_pGdpChangeable)
			{
				GA_PointGroup* pPointGroup=(GA_PointGroup*)m_pGroup;
				if(!m_pIterator)
				{
					m_groupIndex=0;
					GA_Range range=
							m_pGdpChangeable->getPointRange(pPointGroup);
					m_pIterator=new GA_Iterator(range);
				}
				U32 m=m_groupIndex;
				if(m>a_index)
				{
					m_pIterator->rewind();
					m=0;
				}
				for(;m<a_index && m_pIterator->isValid();m++)
				{
					m_pIterator->advance();
				}
				m_groupIndex=m;
			}
			else
			{
				const GA_PointGroup* pPointGroup=(const GA_PointGroup*)m_pGroup;
				if(!m_pIterator)
				{
					m_groupIndex=0;
					GA_Range range=m_pGdp->getPointRange(pPointGroup);
					m_pIterator=new GA_Iterator(range);
				}
				U32 m=m_groupIndex;
				if(m>a_index)
				{
					m_pIterator->rewind();
					m=0;
				}
				for(;m<a_index && m_pIterator->isValid();m++)
				{
					m_pIterator->advance();
				}
				m_groupIndex=m;
			}
#else
			GB_PointGroup& rPointGroup=*(GB_PointGroup*)m_pGroup;
			if(m_pGdpChangeable)
			{
				if(!m_pPointChangeable)
				{
					m_groupIndex=0;
					m_pPointChangeable=
							m_pGdpChangeable->points().head(rPointGroup);
				}
				U32 m=m_groupIndex;
				for(;m>a_index && m_pPointChangeable;m--)
				{
					m_pPointChangeable=
							m_pGdpChangeable->points().prev(
							m_pPointChangeable,rPointGroup);
				}
				for(;m<a_index && m_pPointChangeable;m++)
				{
					m_pPointChangeable=
							m_pGdpChangeable->points().next(
							m_pPointChangeable,rPointGroup);
				}
				m_groupIndex=m;
				m_pPoint=m_pPointChangeable;
			}
			else
			{
				m_pPointChangeable=NULL;

				if(!m_pPoint)
				{
					m_groupIndex=0;
					m_pPoint=m_pGdp->points().head(rPointGroup);
				}
				U32 m=m_groupIndex;
				for(;m>a_index && m_pPoint;m--)
				{
					m_pPoint=m_pGdp->points().prev(m_pPoint,rPointGroup);
				}
				for(;m<a_index && m_pPoint;m++)
				{
					m_pPoint=m_pGdp->points().next(m_pPoint,rPointGroup);
				}
				m_groupIndex=m;
			}
#endif
		}
			break;

		case SurfaceAccessibleI::e_primitiveGroup:
		{
#ifdef FE_HOUDINI_USE_GA
			GA_PrimitiveGroup* pPrimitiveGroup=(GA_PrimitiveGroup*)m_pGroup;
			if(!m_pIterator)
			{
				m_groupIndex=0;
				GA_Range range=m_pGdp->getPrimitiveRange(pPrimitiveGroup);
				m_pIterator=new GA_Iterator(range);
			}
			U32 m=m_groupIndex;
			if(m>a_index)
			{
				m_pIterator->rewind();
				m=0;
			}
			for(;m<a_index && m_pIterator->isValid();m++)
			{
				m_pIterator->advance();
			}
			m_pPrimitive=m_pGdp->getGEOPrimitive(m_pIterator->getOffset());
#else
			GB_PrimitiveGroup& rPrimitiveGroup=*(GB_PrimitiveGroup*)m_pGroup;
			if(!m_pPrimitive)
			{
				m_groupIndex=0;
				m_pPrimitive=m_pGdp->primitives().head(rPrimitiveGroup);
			}
			U32 m=m_groupIndex;
			for(;m>a_index && m_pPrimitive;m--)
			{
				m_pPrimitive=
						m_pGdp->primitives().prev(m_pPrimitive,rPrimitiveGroup);
			}
			for(;m<a_index && m_pPrimitive;m++)
			{
				m_pPrimitive=
						m_pGdp->primitives().next(m_pPrimitive,rPrimitiveGroup);
			}
#endif
			m_groupIndex=m;
		}
			break;

		default:
			;
	}
}

} // namespace
