/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

namespace fe
{

void GroupOp::initialize(void)
{
	catalog<String>("Group")="group";
	catalog<String>("Group","label")="Group Name";
	catalog<String>("Group","page")="Define";

	catalog<String>("Element")="Point";
	catalog<String>("Element","page")="Define";
	catalog<String>("Element","choice:0")="Point";
	catalog<String>("Element","choice:1")="Primitive";

	catalog<bool>("EnableBounds")=true;
	catalog<String>("EnableBounds","label")="Enable";
	catalog<String>("EnableBounds","page")="Bounds";
	catalog<bool>("EnableBounds","joined")=true;

	catalog<bool>("Inverse")=false;
	catalog<String>("Inverse","enabler")="EnableBounds";
	catalog<String>("Inverse","page")="Bounds";

	catalog<String>("Shape")="Box";
	catalog<String>("Shape","enabler")="EnableBounds";
	catalog<String>("Shape","page")="Bounds";
	catalog<String>("Shape","choice:0")="Box";
	catalog<String>("Shape","choice:1")="Sphere";
	catalog<String>("Shape","choice:2")="Surface";
	catalog<String>("Shape","choice:3")="Attribute";

	catalog<SpatialVector>("Center");
	catalog<String>("Center","enabler")="EnableBounds";
	catalog<String>("Center","page")="Bounds";

	catalog<SpatialVector>("Size")=SpatialVector(1.0,1.0,1.0);
	catalog<String>("Size","enabler")="EnableBounds";
	catalog<String>("Size","page")="Bounds";

	catalog< sp<Component> >("Input Surface");

	catalog< sp<Component> >("Bounding Surface");
	catalog<bool>("Bounding Surface","optional")=true;
}

void GroupOp::handle(Record& a_rSignal)
{
	const String groupName=catalog<String>("Group");
	const BWORD inverse=catalog<bool>("Inverse");
	const String shape=catalog<String>("Shape");
	const SpatialVector center=catalog<SpatialVector>("Center");
	const SpatialVector size=catalog<SpatialVector>("Size");

	if(!catalog<bool>("EnableBounds") ||
			size[0]<=0.0 || size[1]<=0.0 || size[2]<=0.0)
	{
		return;
	}

	const BWORD doPrimitive=(catalog<String>("Element")=="Primitive");

	sp<SurfaceAccessorI> spElementPosition;
	if(!accessOutput(spElementPosition,a_rSignal,
			doPrimitive? e_primitive: e_point,e_position)) return;

	sp<SurfaceAccessorI> spElementGroup;
	if(!accessOutput(spElementGroup,a_rSignal,
			doPrimitive? e_primitiveGroup: e_pointGroup,groupName)) return;

	const U32 pointCount=spElementPosition->count();
	if(shape=="Box")
	{
		const SpatialBox box(center-0.5*size,size);
		for(U32 pointIndex=0;pointIndex<pointCount;pointIndex++)
		{
			const SpatialVector point=
					spElementPosition->spatialVector(pointIndex);

			if((intersecting(box,point)) != inverse)
			{
				spElementGroup->append(pointIndex);
			}
		}
	}
	else if(shape=="Sphere")
	{
		for(U32 pointIndex=0;pointIndex<pointCount;pointIndex++)
		{
			const SpatialVector point=
					spElementPosition->spatialVector(pointIndex);

			const SpatialVector scaled=2.0*SpatialVector(
					(point[0]-center[0])/size[0],
					(point[1]-center[1])/size[1],
					(point[2]-center[2])/size[2]);

			if((magnitudeSquared(scaled)<1.0) != inverse)
			{
				spElementGroup->append(pointIndex);
			}
		}
	}
	else if(shape=="Surface")
	{
		sp<SurfaceI> spBounds;
		if(!access(spBounds,"Bounding Surface")) return;

		spBounds->setRefinement(0);

		for(U32 pointIndex=0;pointIndex<pointCount;pointIndex++)
		{
			const SpatialVector point=
					spElementPosition->spatialVector(pointIndex);

			const SurfaceI::Containment containment=
					spBounds->containment(point);

			if((containment==SurfaceI::e_inside) != inverse)
			{
				spElementGroup->append(pointIndex);
			}
		}
	}
	else if(shape=="Attribute")
	{
		sp<SurfaceI> spBounds;
		if(!access(spBounds,"Bounding Surface")) return;

		spBounds->setRefinement(0);

		for(U32 pointIndex=0;pointIndex<pointCount;pointIndex++)
		{
			const SpatialVector point=
					spElementPosition->spatialVector(pointIndex);

			const SurfaceI::Containment containment=
					spBounds->containment(point);

			if((containment==SurfaceI::e_inside) != inverse)
			{
				spElementGroup->append(pointIndex);
			}
		}
	}
}

} // namespace
