# this forge file is for site specific tweaks
import os
import sys

forge = sys.modules["forge"]

def local_setup():
	# parent path for the various versions of Houdini
	# (overriden if the HT environment variable is found)
	forge.houdini_installs = '/opt/'

	# default paths for FE Houdini nodes
	forge.houdini_url_root = 'http://freeelectron.org/op/'
	forge.houdini_icon_root = 'http://freeelectron.org/svg/'

	# desired potential Houdini versions with their associated Boost version
	forge.houdini_versions = [
			[ '11.0.775',		'1_37' ],
			[ '12.0.581',		'1_46' ],
			[ '12.1.125',		'1_46' ],
			[ '12.5.316.22',	'1_51' ],
			[ '12.5.376',		'1_51' ] ]

	# other Boost versions as [version, lib_suffix, include, lib]
	# version is like 1_46, usually blank for the distribution default
	# lib_suffix can be something like '-gcc43-mt-1_51', if needed
	# the distribution lib is usually in /usr/lib or /usr/lib64
	forge.boost_sources = []
	if os.path.exists('/usr/include/boost/exception'):
		forge.cppmap['boost'] = '-DFE_BOOST_EXCEPTIONS'
		forge.boost_sources += [ [ '', '', '/usr/include', '/usr/lib' ] ]

	# parent path for the various versions of Maya
	# (overriden if the MAYA_ROOT environment variable is found)
	forge.maya_installs = '/usr/autodesk/'

	# desired potential Maya versions
	forge.maya_versions = [
			'2012',
			'2013',
			'2014' ]

	# parent paths to "openvdb.h" and "libopenvdb.so"
	# leave blank to not use OpenVDB
	forge.openvdb_include = 'ext/vdb/openvdb'
	forge.openvdb_lib = forge.openvdb_include
