/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __core_AutoHashMap_h__
#define __core_AutoHashMap_h__

namespace fe
{

struct eq_string
{
	bool operator()(const String &s1, const String &s2) const
	{
		return s1 == s2;
	}
};

struct hash_string
{
	FE_UWORD operator()(const String &s) const
	{
		FE_UWORD hash = 0;
		const unsigned char *buffer = s.rawU8();

		for(FE_UWORD i = 0; i < s.length(); i++)
		{
			hash = (hash << 3) + buffer[i];
		}

		return hash;
	}
};

struct eq_type_info
{
	bool operator()(const TypeInfo &t1, const TypeInfo &t2) const
	{
		return t1 == t2;
	}
};

struct hash_type_info
{
	FE_UWORD operator()(const TypeInfo &t) const
	{
		FE_UWORD hash = 0;
		FE_UWORD len = strlen(t.ref().name());
		for(FE_UWORD i = 0; i < len; i++)
			{ hash = (hash << 3) + t.ref().name()[i]; }
		return hash;
	}
};

template<typename T>
struct hash_auto
{
		FE_UWORD	operator()(const T t) const
				{	return reinterpret_cast<FE_UWORD>(t); }
};
template<typename T>
struct eq_auto
{
		bool	operator()(T t1, T t2) const
				{	return t1 == t2; }
};

template<class T>
struct hash_sp
{
		FE_UWORD	operator()(const sp<T>& rspT) const
				{	return reinterpret_cast<FE_UWORD>(rspT.raw()); }
};

template<class T>
struct eq_sp
{
		bool	operator()(const sp<T>& rspT1, const sp<T>& rspT2) const
				{	return rspT1 == rspT2; }
};

template<class T>
struct hash_hp
{
		FE_UWORD	operator()(const hp<T>& rhpT) const
				{	return reinterpret_cast<FE_UWORD>(rhpT.raw()); }
};

template<class T>
struct eq_hp
{
		bool	operator()(const hp<T>& rhpT1, const hp<T>& rhpT2) const
				{	return rhpT1 == rhpT2; }
};

template<typename T, typename U>
class AutoHashMap:
	public HashMap< T, U, hash_auto<T>, eq_auto<T> >						{};

template<typename U>
class AutoHashMap< TypeInfo, U >:
	public HashMap< TypeInfo, U, hash_type_info, eq_type_info >				{};

template<typename U>
class AutoHashMap< String, U >:
	public HashMap< String, U, hash_string, eq_string >						{};

template<typename T, typename U>
class AutoHashMap< sp<T>, U >:
	public HashMap< sp<T>, U, hash_sp<T>, eq_sp<T> >						{};

template<typename T, typename U>
class AutoHashMap< hp<T>, U >:
	public HashMap< hp<T>, U, hash_hp<T>, eq_hp<T> >						{};

} /* namespace */

#endif /* ___core_TypeMaster_h__ */
